"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const crypto_1 = require("crypto");
function createHashKey(namespace, token) {
    return `ACP:${namespace}-${crypto_1.createHash('sha256').update(token).digest('hex')}`;
}
function generateKey(ns, ctx, options) {
    if (!options || (!options.query && !options.headers)) {
        return createHashKey(ns, ctx.path);
    }
    const headers = [];
    if (options.headers) {
        for (const header of options.headers) {
            const value = ctx.get(header);
            if (!value)
                continue;
            headers.push(`${header}:${value}`);
        }
    }
    const queryString = [];
    if (options.query) {
        for (const q of options.query) {
            const value = ctx.query[q];
            if (!value)
                continue;
            queryString.push(`${q}=${value}`);
        }
    }
    const token = ctx.path + (headers.length ? ('-' + headers.join('|')) : '') + (queryString.length ? ('?' + queryString.join('&')) : '');
    return createHashKey(ns, token);
}
exports.generateKey = generateKey;
