"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Debug = require("debug");
const logger = Debug('acp:service:request');
function onSuccessWithCtx(ctx, time) {
    return (response) => {
        const { statusCode, headers, data } = response;
        ctx.statusCode = statusCode;
        const stringData = data;
        for (const key in headers) {
            if (!Object.prototype.hasOwnProperty.call(headers, key))
                continue;
            ctx.set(key, headers[key]);
        }
        ctx.body = stringData;
        logger(`Responded In: ${Date.now() - time} ms`);
        return {
            statusCode,
            headers,
            data: stringData,
        };
    };
}
function onErrorWithCtx(ctx) {
    return (error) => {
        throw error;
    };
}
exports.onErrorWithCtx = onErrorWithCtx;
function makeRequest(ctx, path, headers) {
    logger(`call server for ${path}`);
    const d1 = Date.now();
    const Axios = {};
    return Axios.get(path, { headers }).then(onSuccessWithCtx(ctx, d1), onErrorWithCtx(ctx));
}
exports.makeRequest = makeRequest;
function errorToData(error) {
    const out = { statusCode: 503, data: '', headers: [] };
    try {
        const { message, statusCode, data, headers } = error;
        if (message) {
            return { statusCode, data, headers };
        }
        out.data = message;
    }
    catch (e) { }
    return out;
}
exports.errorToData = errorToData;
